<?php
  function __incException() {
    global $__maxException;
    if(!isset($__maxException)) {
      $__maxException = E_EXCEPTION;
    }
    $__maxException ++;
    return $__maxException;
  }
  define('E_EXCEPTION',1000);
  define('E_UNDEFINED_PROPERTY',__incException());
  define('E_RENDERER_NOT_FOUND',__incException());
  define('E_ACTION_SYNTAX_ERROR',__incException());  
  define('E_ACTION_VARIABLE_NOT_FOUND',__incException());  
  define('E_COMPONENT_NOT_FOUND',__incException());
  define('E_METHOD_NOT_FOUND',__incException());  
  define('E_UNSUPPORTED_RENDERER',__incException());
  define('E_INVALID_DOCTYPE',__incException());
  define('E_ENCODING_NOT_SUPPORTED',__incException());  
  define('E_NO_OUTPUT_HANDLER',__incException());
  define('E_CANNOT_REDIRECT',__incException());
  define('E_CONVERTOR_ERROR',__incException());
  define('E_CONVERTOR_INVALID_DATETIME',__incException());
  define('E_CONVERTOR_INVALID_NUMBER',__incException());
  define('E_CONVERTOR_INVALID_INTEGER',__incException());
  define('E_CONVERTOR_TOO_HIGH',__incException());
  define('E_CONVERTOR_TOO_LOW',__incException());
  define('E_CONVERTOR_TOO_LONG',__incException());
  define('E_CONVERTOR_TOO_SHORT',__incException());
  define('E_FORMAT_CLASS_DOES_NOT_EXIST',__incException());
  define('E_DEFAULT_DATABASE_NOT_DEFINED',__incException());
  define('E_UNDEFINED_EVENT',__incException());
  define('E_EVENT_FUNCTION_NOT_FOUND',__incException());  
  define('E_INLINE_ELEMENT_CANNOT_CONTAIN_BLOCK_ELEMENTS',__incException());
  define('E_DOCTYPE_DOES_NOT_ALLOW_THIS_ELEMENT',__incException());
  define('E_USE_OF_UNKNOWN_ELEMENT',__incException());
  define('E_ELEMENT_SHOULD_BE_SHORT',__incException());
  define('E_NO_AUTH_EVENT_ASSIGNED',__incException());
  define('E_TRANSACTIONS_NOT_SUPPORTED',__incException());
  define('E_NOT_CONNECTED_TO_DATABASE_SERVER',__incException());
  define('E_COULD_NOT_EXECUTE_QUERY',__incException());
  define('E_COULD_NOT_CONNECT_TO_DATABASE_SERVER',__incException());
  define('E_TRANSACTION_ERROR',__incException());
  define('E_COULD_NOT_SELECT_DATABASE',__incException());
  define('E_SERVER_IS_NOT_MULTIDATABASE',__incException());
  define('E_METHOD_MUST_BE_OVERRIDEN',__incException());
  define('E_INCOMPATIBLE_VALUE_FOR_FIELD',__incException());
  define('E_CONNECTION_COULD_NOT_BE_CLOSED',__incException());
  define('E_UNKNOWN_DATA_TYPE_FOR_FIELD',__incException());          
  define('E_CANNOT_INSERT_BLANK_ROW',__incException());
  define('E_FIELD_CANNOT_BE_NULL',__incException());
  define('E_CANNOT_UPDATE_JOINED_TABLE_FIELD',__incException());
  define('E_CALL_FETCH_FIELDS_OR_INSERT_FIRST',__incException());
  define('E_NO_ACTIVE_DATASET',__incException());      
  define('E_FIELD_NOT_FOUND',__incException());
  define('E_VALUE_REQUIRED',__incException());
  define('E_SELECT_BOX_GROUP_NOT_DEFINED',__incException());  
  define('E_COULD_NOT_SET_DB_ENCODING',__incException());
  define('E_INVALID_DB_ENCODING',__incException());
  define('E_CHANGES_COULD_NOT_BE_SAVED',__incException());
  define('E_CLOSING_TAG_OMMITED',__incException());
  define('E_RECORD_KEY_VALUE_MISSING',__incException());
  define('E_RECORD_NOT_FOUND',__incException());  
  define('E_SQL_BAD_SYNTAX_OR_TOO_COMPLEX',__incException());
  define('E_INSERT_MODE_NOT_ALLOWED',__incException());
  define('E_EDIT_MODE_NOT_ALLOWED',__incException());
  define('E_REQUIRED_SETTINGS_MISSING',__incException());
  define('E_RECORD_COULD_NOT_BE_IDENTIFICATED',__incException());
  define('E_TOO_FEW_ITEMS',__incException());
  define('E_TOO_MANY_ITEMS',__incException());
  define('E_INVALID_LOCALE_IDENTIFIER',__incException());
  define('E_RECORD_CANNOT_BE_DELETED',__incException());
  define('E_BROWSE_CAP_DETECTION_FAILED',__incException());
  define('E_DUPLICATE_VALUE',__incException());
  define('E_CANNOT_ADD_AS_CHILD',__incException());
  define('E_LIBRARY_NOT_DEFINED',__incException());          
  /**
  * Information on call to failed function
  */
  class TExceptionTraceInfo extends TObject {
    /** @var string Filename */    
    public $file;
    /** @var int Line number */
    public $line;
    /** @var string Function-name */
    public $function;
    /** @var string Class-name */
    public $class;
    /** @var string Type of call */
    public $type;
    /** @var array Call arguments */
    public $args;

    /**
    * Constructor
    */
    public function __construct() {
      $this->file = '';
      $this->line = 0;
      $this->function = '';            
      $this->class = '';
      $this->type = '';
      $this->args = array();                  
    }      
  }

  /**
  * Base class for every exception throwed TApp framework
  */
  class EException extends Exception {
    /** @var TExceptionTraceInfo Trace information */ 
    public $trace;
    /** @var TCustomLog Loging object */ 
    static public $log = null;    
    /** @var bool Die on exception? */ 
    static public $handleExceptions = false;
    
    /**
    * Output error info to this log
    *
    * @param TCustomLog Loging object
    * @return void
    */  
    public static function setLog(&$log) {
      self::$log = &$log;    
    }
    /**
    * Overridable
    * 
    * @return void
    */    
    protected function log() {
      if(!is_null(self::$log)) {
        self::$log->log('Exception: '.$this->getCode().' '.$this->getMessage().' in '.$this->trace->file.' on line '.$this->trace->line.', call to '.($this->trace->class ? $this->trace->class.'::' : '').$this->trace->function.'( )',E_ERROR);
      }
    }
    /**
    * Constructor
    * 
    * @param mixed Error message or code
    * @param mixed Error Code or array of arguments
    * @param int Level of trace known to be safe
    * @return EException
    */
    public function __construct($messageOrCode = null, $codeOrArgs = 0) {
    
      if(is_int($messageOrCode)) {
        if(is_array($codeOrArgs)) {
          $args = $codeOrArgs;
        } else {
          $args = array();        
        }
        $codeOrArgs = $messageOrCode;
        $messageOrCode = formatError($messageOrCode,$args,false);
      }
      parent::__construct($messageOrCode,$codeOrArgs);
      $this->trace = new TExceptionTraceInfo();
      $this->fetchTraceAtLevel();
      
      $this->log();

      if(EException::$handleExceptions) {

      }           
    }  
   
    /**
    * Returns trace information array at selected level
    *
    * @param int Trace level (default -1)
    * @return array Trace information
    */
    public function fetchTraceAtLevel($level = -1) {
      $trc = $this->getTrace();
      if ($level == -1) {
        while (isset($trc[$level+1])) {
          $level++;
        }
      }
      if($level < 0) {
        $trc = null;
      } else {
        $trc = $trc[$level];                
      }
      
      $this->trace->__construct();
      $this->trace->assignArray($trc);

    }
    /**
    * Returns file where the failed function was called
    *
    * @return string class-name
    */
    public function getCallFile($level = -1) {
      $trc = $this->getTraceAtLevel($level);
      if ($trc == null) {
        return $this->getFile();
      } else {
        return $trc['file'];
      }
    }
    public function getCallLine($level = -1) {
      $trc = $this->getTraceAtLevel($level);
      return $trc['line'];
    }    
  }
  
  EException::$handleExceptions = true;

  function formatError($errorCode, $arguments = array(), $includeErrorCode = false) {
    global $LNG_EXCEPTIONS;
    if($includeErrorCode) {
      $res = '('.$errorCode.') ';
    } else {
      $res = '';
    }
    foreach(array_keys($arguments) as $key) {
      $arguments[$key] = htmlspecialchars($arguments[$key]);
    }
    if(!isset($LNG_EXCEPTIONS[$errorCode])) {
      triggerError('Exception message for error code '.$errorCode.' not found.',E_USER_WARNING);
      return $res.'Unknown exception with arguments ('.implode(',',$arguments).')';
    } else {
      if(count($arguments) > 0) {
        return $res.vsprintf($LNG_EXCEPTIONS[$errorCode],$arguments);
      } else {
        return $res.$LNG_EXCEPTIONS[$errorCode];
      } 
    }
  }
  

  function triggerError($errorMessageOrCode, $argumentsOrLevel = E_USER_ERROR, $level = E_USER_ERROR) {
    if(is_int($argumentsOrLevel)) {
      $aLevel = $argumentsOrLevel;
      $aArguments = array();
    } else {
      $aLevel = $level;
      $aArguments = $argumentsOrLevel;    
    }
    if(is_int($errorMessageOrCode)) {
      $errorMessageOrCode = formatError($errorMessageOrCode,$aArguments);       
    }
//    __roadErrorHandler($aLevel,$errorMessageOrCode);
    trigger_error($errorMessageOrCode,$aLevel);
  }

  function errorLevel($errno) {
    if(($errno & E_ERROR ) || ($errno & E_PARSE ) || ($errno & E_CORE_ERROR ) || ($errno & E_COMPILE_ERROR ) || ($errno & E_USER_ERROR ) || ($errno & E_STRICT ) || ($errno & E_RECOVERABLE_ERROR)) {
      return E_ERROR;
    } elseif(($errno & E_WARNING ) || ($errno & E_CORE_WARNING ) || ($errno & E_COMPILE_WARNING ) || ($errno & E_USER_WARNING )) {
      return E_WARNING;
    } else {
      return E_NOTICE;
    }
  }
  
  function __roadErrorHandler($errno, $errstr, $errfile = '', $errline = 0, $errcontext = array()) {


    $errno = errorLevel($errno);
    
    if(!is_null(EException::$log)) {
      EException::$log->log($errstr.' in '.$errfile.' on line '.$errline,$errno);      
    }

    if(defined('CFG_ADVANCED_ERROR_HANDLER') && !CFG_ADVANCED_ERROR_HANDLER) {
      return false;
    }

    if (!(roadErrorReporting() & $errno)) {
      return;
    }

    if($errno == E_ERROR) {
      global $road;
      if(isset($road)) {
        if(isset($road->output) && ($road->output !== null)) {
          $res = $road->output->renderError($errno, $errstr, $errfile, $errline, $errcontext); 
          if($res !== null) {
            return $res;
          }
        }
      }
      if(defined('CFG_ADVANCED_ERROR_HANDLER') && !CFG_ADVANCED_ERROR_HANDLER) {
        return false;
      }
      global $road;
      if(isset($road)) {
        $road->renderWarnings();
      }
?>
 <div id="roadErrorWindow">
  <div style="position: absolute; width: 100%; height: 100%; top: 0; left: 0; background: black; opacity: .7; -ms-filter:&quot;progid:DXImageTransform.Microsoft.Alpha(Opacity=70)&quot;;	filter: alpha(opacity=70);">&nbsp;</div>
  <div style="font-size: 10pt; font-family: 'Arial'; text-decoration: none; font-weight: normal; background: #FFB7B7; border: 2px solid #5B0000; position: absolute; width: 50%; margin: 0 auto; left: 25%; top: 25%;">
   <div style="background: maroon; border-bottom: 1px solid #5B0000; color: white; font-size: 12pt; padding: 3px 10px; font-weight: bold;"><a href="#" onclick="document.getElementById('roadErrorWindow').style.display = 'none'; return false;" title="Close" style="font-weight: bold; color: white; text-decoration: none; float: right;">X</a>Error occurred</div>
   <div style="padding: 10px;"><?php echo $errstr ?></div>
   <div style="border-top: 1px solid #505050; background: gray; color: white; text-align: left; padding: 3px 10px;"><span style="float: right;"><?php echo htmlspecialchars($errfile) ?></span>line <span style="font-weight: bold"><?php echo htmlspecialchars($errline) ?></span></div>   
  </div>
 </div>
<?php
      die;
    } elseif(($errno == E_WARNING) || ($errno == E_NOTICE)) {
      global $road;
      if(isset($road)) {
        $res = $road->reportWarning($errno, $errstr, $errfile, $errline);
        if($res !== null) {
          return $res;
        }
        if(CFG_ADVANCED_ERROR_HANDLER) {
          return true;
        } else {
          return true;
        }        
      }  
      return false;
    } else {
      return false;
    }
  }
  set_error_handler('__roadErrorHandler');

  function __roadExceptionHandler(/** @var Exception */ $exception) {
    global $road;
    if(isset($road) && isset($road->output) && ($road->output !== null)) {
      $res = $road->output->renderException($exception);
      if($res !== null) {
        return $res;
      }
    }
    $trc = array_reverse($exception->getTrace());    
    if(!is_null(EException::$log)) {
      foreach($trc as $item) {
        EException::$log->log('Exception: '.$exception->getCode().' '.$exception->getMessage().' in '.$item['file'].' on line '.$item['line'].', call to '.(isset($item['class']) ? $item['class'].'::' : '').$item['function'].'( )',E_ERROR);
        break;
      }      
    }
    
    if (!(roadErrorReporting() & E_ERROR)) {
      return;
    }
        
    if(!CFG_ADVANCED_ERROR_HANDLER) {
      return false;
    }
    global $road;
    if(isset($road)) {
      $road->renderWarnings();
    }      
?>
 <div id="roadErrorWindow">
  <div style="position: absolute; width: 100%; height: 100%; top: 0; left: 0; background: black; opacity: .7; -ms-filter:&quot;progid:DXImageTransform.Microsoft.Alpha(Opacity=70)&quot;;	filter: alpha(opacity=70);">&nbsp;</div>
  <div style="z-index: 255; font-size: 10pt; font-family: 'Arial'; text-decoration: none; font-weight: normal; background: #FFB7B7; border: 2px solid #5B0000; position: absolute; width: 50%; margin: 0 auto; left: 25%; top: 25%;">
   <div style="background: maroon; border-bottom: 1px solid #5B0000; color: white; font-size: 12pt; padding: 3px 10px; font-weight: bold;"><a href="#" onclick="document.getElementById('roadErrorWindow').style.display = 'none'; return false;" title="Close" style="font-weight: bold; color: white; text-decoration: none; float: right;">X</a>Uncaught exception<?php echo ($exception->getCode() > 0 ? ' #'.$exception->getCode() : ''); ?></div>
   <div style="padding: 10px;"><?php echo $exception->getMessage() ?></div>
<?php
    if((get_class($exception) == 'EQueryException') || is_subclass_of($exception,'EQueryException')) {
?>
   <div style="border-top: 2px solid #5B0000; background: #505050; color: white; font-weight: bold; padding: 3px 10px;">Last query:</div>
   <div style="font-family: 'Courier New'; font-size: 10pt; padding: 10px 10px;"><?php echo htmlspecialchars($exception->getLastQuery()) ?></div>
<?php
    }
?>
   <div style="border-top: 2px solid #5B0000; background: #505050; color: white; font-weight: bold; padding: 3px 10px;">CallStack:</div>
   <div style="background: gray; color: white; text-align: left; padding: 3px 10px;">
    <ul style="margin: 0; padding: 0; padding-left: 15px; font-size: 7pt;">
<?php
        $i = 0;
        foreach($trc as $key => $item) {
          $i++;        
?>
     <li style="margin: 0; margin-left: 0; padding: 0;<?php if($i < 2) { echo ' border-bottom: 1px solid silver; padding-bottom: 3px; margin-bottom: 3px; font-size: 10pt'; } ?>">
       <div><span style="font-weight: bold;"><?php echo htmlspecialchars($item['line']) ?></span> in <span style="font-weight: bold;"><?php echo htmlspecialchars($item['file']) ?></span> call to <?php if(isset($item['class'])) { echo htmlspecialchars($item['class'].' :: '); } ?><span style="font-weight: bold"><?php echo htmlspecialchars($item['function'])
?></span> ( <a href="#" onclick="if(document.getElementById('roadErrorArgument<?php echo $key ?>').style.display == 'block') { document.getElementById('roadErrorArgument<?php echo $key ?>').style.display = 'none'; } else { document.getElementById('roadErrorArgument<?php echo $key ?>').style.display = 'block' } return false;" style="font-weight: normal; color: blue; text-decoration: underline;">arguments</a> )</div>
       <pre id="roadErrorArgument<?php echo $key ?>" style="display: none;"><?php
//          varInfo($item['args']);
     
?></pre>
     </li>
<?php   } ?>
    </ul>
   </div>   
  </div>
 </div>
<?php
    die;
    return true;
  }
  set_exception_handler('__roadExceptionHandler');
  
  
  function __roadShutDownFunction() {
    $error = error_get_last();
    if ($error['type'] == 1) {
      __roadErrorHandler($error['type'],$error['message'],$error['file'],$error['line']);
      exit;    
    }
  }
  register_shutdown_function('__roadShutdownFunction');
  
  function roadErrorReporting($level = null) {
    global $__roadErrorReporting;
    $res = $__roadErrorReporting;
    if($level !== null) {
      $__roadErrorReporting = $level;
    }
    return $res;
  }
  function __roadDisplayWarnings() {
    
  }
  roadErrorReporting(0);
?>