<?php

 
  /**
  *
  */
  class TMySqlImprovedDataSet extends TDataSet {
    /** @var MySQLi_Result */
    protected $result;
    
    public function __construct(TCustomConnection &$conection, MySQLi_Result &$result, $query) {
      parent::__construct($conection);
      $this->setQuery($query);
      $this->result = &$result;
      
      $fields = $this->result->fetch_fields();
      $i = 0;
      foreach($fields as $field) {
        $this->fieldNames[$field->name] = $i;
        $i++;        
      }
    }
    protected function doFetchField($index) {
      $fieldDef = $this->result->fetch_field_direct($index);
      $field = new TField($this);      
      $field->name = $fieldDef->name;
      $field->length = $fieldDef->length;      
      $field->table = $fieldDef->table;
      $field->orgTable = $fieldDef->orgtable;
      $field->decimals = $fieldDef->decimals;
      $field->defaultValue = $fieldDef->def;
      $field->orgName = $fieldDef->orgname;
      $field->orgName = $fieldDef->orgname;      
      $field->nativeDataType = $fieldDef->type;

      if(($field->nativeDataType == MYSQLI_TYPE_TINY) && ($field->length == 1)) {
        $field->setDataType(TYPE_BOOLEAN);
      } elseif(arrayContains($field->nativeDataType,
        array(MYSQLI_TYPE_INT24,MYSQLI_TYPE_LONG,MYSQLI_TYPE_LONGLONG,MYSQLI_TYPE_SHORT,MYSQLI_TYPE_TINY)
      )) {
        $field->setDataType(TYPE_INTEGER);
        if($fieldDef->flags & MYSQLI_UNSIGNED_FLAG) {
          $field->value->minValue = 0;
        }
      } elseif(arrayContains($field->nativeDataType,
        array(MYSQLI_TYPE_DECIMAL,MYSQLI_TYPE_DOUBLE,MYSQLI_TYPE_FLOAT,MYSQLI_TYPE_NEWDECIMAL,MYSQLI_TYPE_YEAR)
      )) {
        $field->setDataType(TYPE_DECIMAL);
        if($fieldDef->flags & MYSQLI_UNSIGNED_FLAG) {
          $field->value->minValue = 0;
        }
      } elseif(arrayContains($field->nativeDataType,
        array(MYSQLI_TYPE_ENUM,MYSQLI_TYPE_CHAR,MYSQLI_TYPE_STRING,MYSQLI_TYPE_VAR_STRING)
      )) {
        $field->setDataType(TYPE_STRING); 
        $field->value->maxLength = round($field->length / 3);
      } elseif(arrayContains($field->nativeDataType,
        array(MYSQLI_TYPE_BLOB,MYSQLI_TYPE_LONG_BLOB,MYSQLI_TYPE_MEDIUM_BLOB,MYSQLI_TYPE_TINY_BLOB)
      )) {
        $field->setDataType(TYPE_MEMO);
      } elseif(arrayContains($field->nativeDataType,
        array(MYSQLI_TYPE_DATE)
      )) {
        $field->setDataType(TYPE_DATE);
      } elseif(arrayContains($field->nativeDataType,
        array(MYSQLI_TYPE_DATETIME,MYSQLI_TYPE_TIMESTAMP)
      )) {
        $field->setDataType(TYPE_DATETIME);
      } elseif(arrayContains($field->nativeDataType,
        array(MYSQLI_TYPE_TIME)
      )) {
        $field->setDataType(TYPE_TIME);
      } else {
        $field->setDataType(TYPE_UNKNOWN);      
      }
      $field->autoValue = ($fieldDef->flags & MYSQLI_AUTO_INCREMENT_FLAG);
      $field->primaryKey = ($fieldDef->flags & MYSQLI_PRI_KEY_FLAG);
      $field->notNull = ($fieldDef->flags & MYSQLI_NOT_NULL_FLAG);
      $field->unique = ($fieldDef->flags & MYSQLI_UNIQUE_KEY_FLAG);
      $field->index = $index;
      return $field;
    }
    public function __destruct() {
      if($this->freeOnRelease) {
        $this->result->free();
      }
    }
    public function getRowCount() {
      return $this->result->num_rows;
    }
    public function getFieldCount() {
      return $this->result->field_count;
    }
    protected function doSeekTo(&$rowIndex) {
      return $this->result->data_seek($rowIndex);
    }
    protected function doFetchObject() {
      return $this->result->fetch_object();
    }
    protected function doFetchArray() {
      return $this->result->fetch_row();
    }
    protected function doFetchAssoc() {
      return $this->result->fetch_assoc();
    }
    public function fetchFields() {
      // PROBLEM:  when ordered by field from connected table,
      //           the primary table looses its PK reference
      // SOLUTION: dummy 0 record result fetched for field object
      //           generating, while ORDER BY beeing stripped away            
      if(!is_array($this->fields)) {
        $q = sqlExplode($this->getQuery());
        if(isset($q['ORDER BY'])) {
          $prior = $this->result;
          unset($q['ORDER BY']);
          $q['WHERE'] = '1 = 0';          
          $this->result = $this->connection->getNativeDriver()->query(sqlImplode($q));

          if(!is_object($this->result)) {
            echo sqlImplode($q);
            $this->result = $prior;
            unset($prior);
          }          
        }  
      }

      if(isset($prior)) {
        $res = parent::fetchFields();
        $this->result = $prior;
        return $res;        
      } else {
        return parent::fetchFields(); 
      }
    }
  }
  
  $__roadMySqliCharsetMatrix = array(
    'utf-([0-9]+)' => 'utf$1',
    'windows-([0-9]+)' => 'CP$1',
    'iso-8859-([0-9]+)' => 'latin$1'
  );
  /**
  *
  */
  class TMySqlImprovedConnection extends TCustomConnection {
    /** @var MySQLi */
    protected $mysqli = null;
    /** @var TSqlConnectionParameters Connection parameters */
    public $server;
    /**
    * Returns the MySQLi object
    *
    * @return MySQLi
    */
    public function getNativeDriver()  {
      return $this->mysqli;
    }
    protected function setOutputEncoding($encoding) {
      global $__roadMySqliCharsetMatrix;
      foreach($__roadMySqliCharsetMatrix as $orig => $new) {
        $count = 0;
        $res = preg_replace('/'.$orig.'/',$new,$encoding,1,$count);
        if($count == 1) {
          break;
        }
        $res = false;
      }
      if($res === false) {
        $this->setLastError(E_INVALID_DB_ENCODING,formatError(E_INVALID_DB_ENCODING,array($encoding)));
        return false;
      }      
      if(!$this->mysqli->set_charset($res)) {
        $this->setLastError($this->mysqli->errno,$this->mysqli->error);
        return false;        
      }
      return true;      
    }
    public function getInsertId() {
      return $this->mysqli->insert_id;
    }    
    protected function getLastConnectError(&$code,&$message) {
      $code = $this->mysqli->connect_errno;
      $message = $this->mysqli->connect_error;
    }
    protected function getLastQueryError(&$code,&$message) {    
      $code = $this->mysqli->error;
      $message = $this->mysqli->errno;
    }
    protected function doConnect() {
      $res = $this->mysqli->real_connect($this->server->host,$this->server->username,$this->server->password, null, $this->server->port);
      $this->setLastError($this->mysqli->connect_errno,$this->mysqli->connect_error);
      $this->mysqli->autocommit(true);       
      return $res; 
    }
    protected function doQuery($query, $fetchResult) {
      $res = $this->checkConnected();
      if($res) {
        $res = ($result = $this->mysqli->query($query));
        if($res) {
          if($fetchResult) {
            $this->result = null;
            $this->result = new TMySqlImprovedDataSet($this,$result,$query);
          }
        } else {
          $this->setLastError($this->mysqli->errno,$this->mysqli->error);
        }
      }
      return $res;

    }
    public function __construct() {
      parent::__construct();
      $this->multiDatabase = true;
      $this->transactionsSupported = true;
      $this->server = new TSqlConnectionParameters();
      $this->mysqli = mysqli_init();
    }
    protected function doGetClientInfo() {
      return $this->mysqli->client_info;
    }
    protected function doGetClientVersion() {
      return $this->mysqli->client_version;
    }
    protected function doGetServerInfo() {
      return $this->mysqli->server_info;
    }
    protected function doGetServerVersion() {
      return $this->mysqli->server_version;
    }
    protected function doClose() {
      $res = $this->mysqli->close();
      if(!$res) {
        $this->setLastError($this->mysqli->errno,$this->mysqli->error);
      }
      return $res;
    }
    public function getMatchedCount() {
      if (preg_match('/Rows matched: ([0-9]*)/',$this->mysqli->info,$arg)) {
        return $arg[1]+0;
      } else {
        return 0;
      }
    }
    public function getAffectedCount() {
      return $this->mysqli->affected_rows;
    }

  }



?>