<?php
  class TXMLOutput extends TCustomOutput {
//    public $contentType = 'application/json';
    public $buffered = true;
    /** @var TXMLPage */
    public $page;  
    public $contentType = 'application/xml';
    private function getTagCode($tagName, $attributes = null, $closed)  {
      $code = '<'.$tagName;
      if($attributes) {
        if(is_array($attributes)) {
          foreach($attributes as $name => $value) {
            if($value === false) {
              $value = 'false';
            }          
            if($value === true) {
              $value = 'true';
            }
            $code .= ' '.$name.'="'.htmlspecialchars($value).'"';
          }
        } else {
          $code .= ' '.$attributes;
        }
      }
      if($closed) {
        $code .= ' /';
      }
      $code .= '>';
      return $code;
    }
    public function warningReported($warning, $warningCount)  {
      return true;
    }
    
    public function renderError($errno, $errstr, $errfile = '', $errline = 0, $errcontext = array()) {
      $this->buffered = false;
      $this->page->renderHeader($this->road->page,'error');
      $this->rTagText('internalError',$errstr,array('code' => $errno));
      $this->page->renderFooter($this->road->page,'error');            
      exit;
    }

    public function renderException($exception) {
      $this->renderError($exception->getCode(),
        $exception->getMessage(),
        $exception->getFile(),
        $exception->getLine()
      );
    }
    public function rCode($code)  {
      $this->write($code);
    }
    public function escapeText($text) {
      return htmlspecialchars($text);
    }
    public function rText($code)  {
      $this->write($this->escapeText($code));
    }    
    /**
    * Render HTML start tag
    *
    * @param string Tag name
    * @param string[] Attribute list
    * @param bool indent the code
    * @param bool put the code on single line                
    * @return void
    */
    public function rTagOpen($tagName, $attributes = null)  {
      $this->rCode($this->getTagCode($tagName,$attributes,false));      
    }
    
    /**
    * Render a short HTML tag
    *
    * @param string Tag name
    * @param string[] Attribute list
    * @param bool indent the code
    * @param bool put the code on single line                
    * @return void
    */
    public function rTagShort($tagName, $attributes = null)  {
      $tt = $this->getTagType($tagName);
      $this->handleIndentAndLine($indented,$line,$tt);

      $code = $this->getTagCode($tagName,$attributes,true);

      $this->rCode($code,$indented,$line);   
    }    

    /**
    * Render ending HTML tag
    *
    * @param string Tag name
    * @param bool indent the code
    * @param bool put the code on single line
    * @return void
    */
    public function rTagClose($tagName)  {      
      $code = '</'.$tagName.'>';
      $this->rCode($code);
    }
    /**
    *
    *
    * @return void
    */
    public function rTagText($tagName, $text, $attributes = null)  {
      $this->rCode($this->getTagCode($tagName,$attributes,false)
        .$this->escapeText($text).'</'.$tagName.'>'
      );
    }
    
  }
  class TXMLRenderer extends TCustomRenderer {
    /** @var TXMLOutput */
    public $output;
    protected function getAttributes($component) {
      return array('id' => $component->getId());
    }
    protected function getTag($component) {
      return strDelete($component->getClass(),0,1);
    }    
  }
  class TXMLNotable extends TXMLRenderer {
    public function renderHeader(TComponent $component, $flags = null) {
      $this->output->rTagOpen($this->getTag($component),$this->getAttributes($component));
    }
    public function renderFooter(TComponent $component, $flags = null) {
      $this->output->rTagClose($this->getTag($component));
    }    
  }
  class TXMLControl extends TXMLNotable {    
    protected function getAttributes($component) {
      $attributes = parent::getAttributes($component);
      $attributes['caption'] = $component->getCaption();
      return $attributes;
    }
  }
  class TXMLPage extends TXMLNotable {    
    protected function getTag($component) {
      return 'roadApi';
    }
    protected function getAttributes($component) {
      $attributes = parent::getAttributes($component);
      $attributes['title'] = $component->title;
      return $attributes;
    }    
    public function renderHeader(TComponent $component, $flags = null) {
      $this->output->rCode('<?xml version="1.0" encoding="'.$this->output->escapeText($this->output->encoding).'" ?>');      
      parent::renderHeader($component,$flags);
      if(($flags <> 'error') && $this->road->hasMessages(MSG_SUCCESS | MSG_ERROR | MSG_HIGHLIGHT)) {
        $this->output->rTagOpen('statusMessages');
        foreach($this->road->dequeueMessages(MSG_SUCCESS) as $msg) {
          $this->output->rTagText('success',$msg);
        }
        foreach($this->road->dequeueMessages(MSG_ERROR) as $msg) {
          $this->output->rTagText('error',$msg);
        }
        foreach($this->road->dequeueMessages(MSG_HIGHLIGHT) as $msg) {
          $this->output->rTagText('highlight',$msg);
        }
        $this->road->clearMessages();        
        $this->output->rTagClose('statusMessages');        
      }
//      $this->output->rTagOpen($this->getTag($component),array('id'=>$component->getId()));
    }    
  }
  class TXMLFieldLinkedControl extends TXMLControl {
    /** @param TInputControl */    
    protected function getAttributes($component) {
      $attributes = parent::getAttributes($component);
      if($f = $component->getFormat()) {
        $attributes['format'] = strDelete($f->getClass(),0,1);
      }
      return $attributes;
    }
    public function renderChildren(TComponent $component, $flags = null) {
      $this->output->rText($component->getText());
    }
  }
  class TXMLStaticText extends TXMLFieldLinkedControl {
    protected function getAttributes($component) {
      $attributes = parent::getAttributes($component);
      unset($attributes['caption']);
      $types = array(
        TEXT_BLOCK => 'block',
        TEXT_HEADER_1 => 'header1',
        TEXT_HEADER_2 => 'header2',
        TEXT_HEADER_3 => 'header3',
        TEXT_HEADER_4 => 'header4',
        TEXT_HEADER_5 => 'header5',
        TEXT_HEADER_6 => 'header6',
        TEXT_INLINE => 'inline',
        TEXT_PARAGRAPH => 'paragraph',
        TEXT_PREFORMATED => 'preformated'
      );
      if(isset($types[$component->kind])) {
        $attributes['kind'] = $types[$component->kind];
      }
      return $attributes;
    }
  }
  class TXMLSelectBox extends TXMLFieldLinkedControl {
    /** @param TSelectBox */
    public function renderChildren(TComponent $component, $flags = null) {    
      foreach($component->options as /** @var TSelectOption */ $opt) {
        $attr = array('value' => $opt->value);
        if($opt->isSelected()) {
          $attr['selected'] = true;
        } 
        $this->output->rTagText('option',$opt->getCaption(),$attr);
      }
    }  
  }    
  $this->output = new TXMLOutput($this);
  $this->output->page = new TXMLPage($this);
  $this->output->type = OUTPUT_XML;  
  $this->output->registerRenderer(new TXMLRenderer($this),'TComponent');
  $this->output->registerRenderer($this->output->page,'TPage');        
  $this->output->registerRenderer(new TXMLFieldLinkedControl($this),'TFieldLinkedControl');
  $this->output->registerRenderer(new TXMLControl($this),'TRelationControl');
  $this->output->registerRenderer(new TXMLSelectBox($this),'TSelectBox');
  $this->output->registerRenderer(new TXMLSelectBox($this),'TMultiSelectBox');
  $this->output->registerRenderer(new TXMLStaticText($this),'TStaticText');  
?>