<?php

  class TXhtmlUserPageRenderer extends TXhtmlPage {

    public function renderHeader(TComponent $component,$flags = null)  {
      // make sure header has not been rendered yet
      if($this->headerRendered) return;
      // render <head> section
      parent::renderHeader($component,$flags);
      // do not draw anything if the browser has not been detected yet 
      if($flags == 'undetectedBrowser') return;

      /* Here you can draw your divs, headers etc. */
      $this->output->rTagText('h1',CFG_APPLICATION_NAME);

      $this->output->rTagOpen('div',array('class'=>'menu'));
      $this->output->rLinkText('/index.php','Hlavní stránka',array('class'=>'rButton'));
      $this->output->rLinkText('/seznam-vyletu.php','Všechny výlety',array('class'=>'rButton'));
      if($this->road->security->isAllowed(99)) {
        $this->output->rLinkText('/sluzby.php','Seznam služeb',array('class'=>'rButton'));
      }
      $this->output->rLinkText('/login.php','Přihlášení',array('class'=>'rButton'));      
      $this->output->rTagClose('div');

      // render success, error and information messages
      $this->output->renderMessages();
    }

    public function renderFooter(TComponent $component,$flags = null)  {
      // make sure footer has not been rendered yet
      if($this->footerRendered) return;
      // do not draw anything if the browser has not been detected yet 
      if($flags !== 'undetectedBrowser') { 

        /* Here you can draw your copyrights, cross-links and validity proofs. */
  
        $this->output->rTagOpen('div',array('class'=>'footer'));
        // spread the ROAD across the universe
        $this->output->rRunningOnTheRoad();
        $this->output->rCode(
          'vygenerováno za <strong>'.number_format($this->road->getExecutionTime(),3,',','').'</strong> s'
        );
        $this->output->rTagClose('div');
      }
      // end <body> and <html> tags       
      parent::renderFooter($component,$flags);      
    }

    public function beforeRender(TComponent $component, $flags = null) {
      parent::beforeRender($component,$flags);

      /* This is a proper place to link all needed style and script files. */
      $this->metaAddStyle('/'.relativePath(dirname(__FILE__)).'/style.css');
      //$this->metaAddScript('/'.relativePath(dirname(__FILE__)).'/script.js');
    }
  }

  // register our renderer to override standard page rendering
  //     note that we are in context of TRoad::init() method,
  //     so please use $this instead of $road global variable
  $this->output->registerRenderer(new TXhtmlUserPageRenderer($this),'TPage');


  class TXhtmlVylet extends TXhtmlRenderer {
    function renderContent(TComponent $component,$flags = null) {
      $this->output->rTagOpen('div',$this->getAttributes($component));
      $this->output->rTagText('h3',$component->dataset->getAsDisplay('nazev'));
      if($component->dataset->hasValue('obrazek')) {
        $this->output->rImg($component->dataset->getAsSql('obrazek'),$component->dataset->getAsDisplay('nazev'));
      }
      if($component->dataset->hasValue('souradnice_ss') && $component->dataset->hasValue('souradnice_vd')) {
        $this->output->rLinkText('http://maps.google.com/maps?q='.$component->dataset->getAsSql('souradnice_ss').','.$component->dataset->getAsSql('souradnice_vd'),'na mapě',array('class'=>'naMape'));
      }
      $this->output->rTagText('p',implode(', ',$component->sluzby),array('class'=>'sluzby'));
      $this->output->rTagText('p',$component->dataset->getAsDisplay('popis'),array('class'=>'popis'));
      $this->output->rTagText('div',$component->dataset->getAsDisplay('cena').' Kč',array('class'=>'cena'));
      $this->output->rTagText('div',$component->dataset->getAsDisplay('jmeno'),array('class'=>'vedouci'));
      $this->output->rTagClose('div');
    }
  }
  $this->output->registerRenderer(new TXhtmlVylet($this),'TVylet');
?>