<?php

  /**
  * Parses Accept-* header and returns associative array of accepted values and their quality
  *
  * @param string content encoding
  * @return float[]
  */  
  function parseAcceptHeader($headerValue) {
    $x = explode(',',$headerValue);
    $res = array();
    foreach($x as $item) {
      if(preg_match('/^ *([^ ,;]+)(; *q=[0-9\.]+)? *$/',$item,$matches) !== false) {
        if(isset($matches[2])) {
          $q = substr($matches[2],3)+0;
        } else {
          $q = 1;
        }
        $res[strtolower($matches[1])] = $q;
      }
    }
    return $res;    
  }

  function __getAcceptValueQuality(&$array, $value) {
    if(count($array) == 0) {
      return 1;
    }
    $value = strtolower($value);
    if(isset($array[$value])) {
      return $array[$value];
    } elseif(isset($array['*'])) {
      return $array['*'];
    } else {
      return false;
    }
  }


  /**
  * Does the browser accept selected content encoding
  *
  * @param string content encoding
  * @return mixed False if not q value otherwise
  */
  function contentEncodingAccepted($value)  {
    global $_ACCEPT_ENCODING;
    return __getAcceptValueQuality($_ACCEPT_ENCODING,$value);
  }

  /**
  * Does the browser accept selected charset encoding
  *
  * @param string charset encoding
  * @return bool
  */
  function charsetEncodingAccepted($value)  {
    global $_ACCEPT_CHARSET;
    return __getAcceptValueQuality($_ACCEPT_CHARSET,$value);
  }

  /**
  * Does the browser accept selected content type
  *
  * @param string content type
  * @return bool
  */
  function contentTypeAccepted($value)  {
    global $_ACCEPT_CONTENT_TYPE;
    return __getAcceptValueQuality($_ACCEPT_CONTENT_TYPE,$value);
  }

  /**
  * Return array containing information about technologies supported by browser
  *
  * @return obj
  */
  function getBrowserCapabilities() {
    if (get_cfg_var('browscap') && !CFG_ALLWAYS_USE_EXTERNAL_BROWSCAP) {
      $res = get_browser(null,true); //If available, use PHP native function
      if(!is_array($res)) {
        $res = array();
        triggerError(E_BROWSE_CAP_DETECTION_FAILED,E_USER_WARNING);
      }      
    } else {
      require_once(CFG_ROAD_DIR.'/lib/browsecap/Browscap.php');
      if(!is_dir(CFG_ROAD_DIR.'/cache/browscap')) {
        mkdir(CFG_ROAD_DIR.'/cache/browscap');
      }
      $caps = new Browscap(CFG_ROAD_DIR.'/cache/browscap');
      if(CFG_REFRESH_BROWSECAP_INI_EVERY) { 
        $caps->doAutoUpdate = true;
        $caps->updateInterval = CFG_REFRESH_BROWSECAP_INI_EVERY*86400;
      } else {
        $caps->doAutoUpdate = false;
      }
      try {
        $x = $caps->getBrowser(null,true);
      } catch(Exception $e) {
        triggerError(E_BROWSE_CAP_DETECTION_FAILED,E_USER_WARNING);
      }      
      $res = array();
      if(is_array($x)) {
        foreach($x as $key => $value) {
          $res[strtolower($key)] = $value;
        }
      } else {
        triggerError(E_BROWSE_CAP_DETECTION_FAILED,E_USER_WARNING);
      }
    }
    return $res;
  }
  
  if(!function_exists('getallheaders')) {
    function getallheaders() {
      $headers = array();
      foreach($_SERVER as $h=>$v) {
        if(ereg('HTTP_(.+)',$h,$hp)) {
          $headers[$hp[1]]=$v;
        }
      }
      return $headers;
    }  
  }    
?>