<?php
  define('CORNER_FILE','');
  /**
  * Loads image from file of any supported type
  *
  * @param string Filename
  * @return resource GD image
  */
  function imagecreatefromfile($filename) {
    $ext = pathinfo($filename);
    if(isset($ext['extension'])) {
      switch(strtolower($ext['extension'])) {
        case 'jpg' || 'jpeg': return imagecreatefromjpeg($filename);
        case 'png': return imagecreatefrompng($filename);
        case 'gif': return imagecreatefromgif($filename);
        case 'bmp': return imagecreatefromwbmp($filename);
        case 'xbm': return imagecreatefromxbm($filename);
        case 'xpm': return imagecreatefromxpm($filename);
      }
    }
    throw new Exception('Unsupported image type: '.$filename);
  }

  /**
  * Masks source image with white round corners
  *
  * @param resource GD image
  * @param bool Make Top-Left corner?
  * @param bool Make Bottom-Left corner?
  * @param bool Make Bottom-Right corner?
  * @param bool Make Top-Right corner?
  * @return void
  */
  function imagemakecorners($img,$topleft = true, $bottomleft = true, $bottomright = true, $topright = true) {
    if(!is_file(CORNER_FILE)) {
      throw new Exception('Corner file not found');
    }
    $corner = imagecreatefrompng(CORNER_FILE);
    $ih = imagesy($img);
    $iw = imagesx($img);
    $ch = imagesy($corner);
    $cw = imagesx($corner);
    $hh = floor($ch/2);
    $hw = floor($cw/2);

    if($topleft) {
      imagecopymerge_alpha($img,$corner,0,0,0,0,$hw,$hh,100);
    }
    if($topright) {
      imagecopymerge_alpha($img,$corner,$iw-$hw,0,$cw-$hw,0,$hw,$hh,100);
    }
    if($bottomleft) {
      imagecopymerge_alpha($img,$corner,0,$ih-$hh,0,$ch-$hh,$hw,$hh,100);
    }
    if($bottomright) {
      imagecopymerge_alpha($img,$corner,$iw-$hw,$ih-$hh,$cw-$hw,$ch-$hh,$hw,$hh,100);
    }
    imagedestroy($corner);
  }
  /**
  * Alpha-Chanel enabled imagecopymerge
  *
  * @param resource Destination image
  * @param resource Source image
  * @return bool Successfull?
  */
  function imagecopymerge_alpha($dst_im, $src_im, $dst_x, $dst_y, $src_x, $src_y, $src_w, $src_h, $pct, $trans = NULL)
  {
    $dst_w = imagesx($dst_im);
    $dst_h = imagesy($dst_im);
  
    // bounds checking
    $src_x = max($src_x, 0);
    $src_y = max($src_y, 0);
    $dst_x = max($dst_x, 0);
    $dst_y = max($dst_y, 0);
    if ($dst_x + $src_w > $dst_w)
      $src_w = $dst_w - $dst_x;
    if ($dst_y + $src_h > $dst_h)
      $src_h = $dst_h - $dst_y;
  
    for($x_offset = 0; $x_offset < $src_w; $x_offset++)
      for($y_offset = 0; $y_offset < $src_h; $y_offset++)
      {
        // get source & dest color
        $srccolor = imagecolorsforindex($src_im, imagecolorat($src_im, $src_x + $x_offset, $src_y + $y_offset));
        $dstcolor = imagecolorsforindex($dst_im, imagecolorat($dst_im, $dst_x + $x_offset, $dst_y + $y_offset));
  
        // apply transparency
        if (is_null($trans) || ($srccolor !== $trans))
        {
          $src_a = $srccolor['alpha'] * $pct / 100;
          // blend
          $src_a = 127 - $src_a;
          $dst_a = 127 - $dstcolor['alpha'];
          $dst_r = ($srccolor['red'] * $src_a + $dstcolor['red'] * $dst_a * (127 - $src_a) / 127) / 127;
          $dst_g = ($srccolor['green'] * $src_a + $dstcolor['green'] * $dst_a * (127 - $src_a) / 127) / 127;
          $dst_b = ($srccolor['blue'] * $src_a + $dstcolor['blue'] * $dst_a * (127 - $src_a) / 127) / 127;
          $dst_a = 127 - ($src_a + $dst_a * (127 - $src_a) / 127);
          $color = imagecolorallocatealpha($dst_im, $dst_r, $dst_g, $dst_b, $dst_a);
          // paint
          if (!imagesetpixel($dst_im, $dst_x + $x_offset, $dst_y + $y_offset, $color))
            return false;
          imagecolordeallocate($dst_im, $color);
        }
      }
    return true;
  }
  /**
  * Resize image to fit max boundaries
  *
  * @param resource GD Image
  * @param int Max width
  * @param int Max height  
  * @return bool Was the resizing needed?
  */
  function imageresize(&$img,$maxWidth,$maxHeight) {
    $source = $img;
    $sw = imagesx($source);
    $sh = imagesy($source);
    if(($sw <= $maxWidth) && ($sh <= $maxHeight)) {
      return false;
    }
    if((($sw - $maxWidth)/$maxWidth) >= (($sh - $maxHeight)/$maxHeight)) {
      $dw = $maxWidth;
      $dh = round($dw*($sh/$sw));
    } else {
      $dh = $maxHeight;
      $dw = round($dh*($sw/$sh));
    }
    $img = imagecreatetruecolor($dw,$dh);
    if(!imagecopyresampled($img,$source,0,0,0,0,$dw,$dh,$sw,$sh)) {
      throw new Exception('Could not resample the image');
    }
    return true;
  }
?>